package zinger.jrf;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.text.JTextComponent;

public class Console extends JFrame implements ActionListener
{
    private static final String TEXT_AREA_PATH_DELIMITER = File.pathSeparator + "\n";

    protected JTextField resourceNameField;
    protected JTextComponent pathArea;
    protected JButton goButton;
    protected JTextComponent outputArea;

    protected final JavaResourceFinder finder;

    public Console()
    {
        this(new JavaResourceFinder());
    }

    public Console(final JavaResourceFinder finder)
    {
        super("Java Resource Finder");

        this.finder = finder;
    }

    protected void frameInit()
    {
        super.frameInit();

        this.resourceNameFieldInit();
        this.pathAreaInit();
        this.buttonInit();
        this.outputAreaInit();

        final Font font = new Font("Courier", 0, 14);
        this.resourceNameField.setFont(font);
        this.pathArea.setFont(font);
        this.outputArea.setFont(font);

        this.menuInit();
        this.setContentPane(this.createContentPane());
    }

    protected void menuInit()
    {
        final JMenuItem parseResourcePathsItem = new JMenuItem("Parse Resource Paths");
        parseResourcePathsItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent ev)
            {
                Console.this.parseResourcePaths();
            }
        });

        final JMenuItem exitMenuItem = new JMenuItem("Exit");
        exitMenuItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent ev)
            {
                System.exit(0);
            }
        });

        final JMenu actionMenu = new JMenu("Actions");
        actionMenu.add(parseResourcePathsItem);
        actionMenu.add(exitMenuItem);

        final JMenuBar menuBar = new JMenuBar();
        menuBar.add(actionMenu);
        this.setJMenuBar(menuBar);
    }

    protected Container createContentPane()
    {
        final JPanel inputPane = new JPanel(new BorderLayout());
        inputPane.add(this.resourceNameField, BorderLayout.CENTER);
        inputPane.add(this.goButton, BorderLayout.EAST);

        final JPanel contentPane = new JPanel(new BorderLayout());
        contentPane.add(inputPane, BorderLayout.NORTH);
        contentPane.add(new JScrollPane(this.pathArea), BorderLayout.CENTER);
        contentPane.add(this.outputArea, BorderLayout.SOUTH);

        return contentPane;
    }

    protected void resourceNameFieldInit()
    {
        this.resourceNameField = new JTextField();
        this.resourceNameField.addActionListener(this);
        this.resourceNameField.setToolTipText("resource name");
    }

    protected void pathAreaInit()
    {
        this.pathArea = new JTextArea();
        ((JTextArea)this.pathArea).setLineWrap(true);
        this.pathArea.setToolTipText("resource path list");
    }

    protected void buttonInit()
    {
        this.goButton = new JButton("Go");
        this.goButton.addActionListener(this);
    }

    protected void outputAreaInit()
    {
        this.outputArea = new JTextArea();
        ((JTextArea)this.outputArea).setLineWrap(true);
        this.outputArea.setEditable(false);
    }

    public void actionPerformed(final ActionEvent ev)
    {
        final Object source = ev.getSource();
        if(source == this.resourceNameField || source == this.goButton)
        {
            this.runSearch();
        }
    }

    public Dimension getPreferredSize()
    {
        return new Dimension(600, 400);
    }

    protected void runSearch()
    {
        finder.setResourcePath(this.pathArea.getText(), Console.TEXT_AREA_PATH_DELIMITER);

        this.outputArea.setText("Searching...");

        final StringBuffer sb = new StringBuffer();
        try
        {
            for(final Enumeration enum = finder.getResources(this.resourceNameField.getText()); enum.hasMoreElements();)
            {
                sb.append(enum.nextElement()).append('\n');
            }
        }
        catch(final IOException ex)
        {
            sb.append(ex);
        }

        this.outputArea.setText(sb.toString());
    }

    protected void parseResourcePaths()
    {
        final StringBuffer sb = new StringBuffer();
        for(final StringTokenizer st = new StringTokenizer(this.pathArea.getText(), Console.TEXT_AREA_PATH_DELIMITER); st.hasMoreTokens();)
        {
            sb.append(st.nextToken()).append('\n');
        }
        this.pathArea.setText(sb.toString());
    }

    public static void main(final String[] args)
    {
        final Console console = new Console();
        console.addWindowListener(new WindowAdapter()
        {
            public void windowClosing(final WindowEvent ev)
            {
                System.exit(0);
            }
        });
        console.pack();
        console.setVisible(true);
    }
}
