package zinger.jrf;

import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.PrintStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Enumeration;
import java.util.StringTokenizer;

public class JavaResourceFinder
{
    static final String PROMPT = "jrf> ";

    private ClassLoader loader = ClassLoader.getSystemClassLoader();

    public void setResourcePath(final String resourcePath)
    {
        this.setResourcePath(resourcePath, File.pathSeparator);
    }

    public void setResourcePath(final String resourcePath, final String pathDelimiters)
    {
        final StringTokenizer tokenizer = new StringTokenizer(resourcePath, pathDelimiters);
        final URL[] urls = new URL[tokenizer.countTokens()];
        for(int i = 0; i < urls.length; i++)
        {
            try
            {
                urls[i] = (new File(tokenizer.nextToken())).toURL();
            }
            catch(final MalformedURLException ex)
            {
                ex.printStackTrace();
            }
        }
        this.setLoader(new URLClassLoader(urls));
    }

    protected void setLoader(final ClassLoader loader)
    {
        this.loader = loader;
    }

    public Enumeration getResources(final String name) throws IOException
    {
        return this.loader.getResources(name);
    }

    public void listResources(final String name, final PrintStream out) throws IOException
    {
        for(final Enumeration enum = this.getResources(name); enum.hasMoreElements();)
        {
            out.println(enum.nextElement());
        }
    }

    public static void main(final String[] args)
    {
        final JavaResourceFinder finder = new JavaResourceFinder();
        if(args.length > 0)
        {
            for(int i = 0; i < args.length; i++)
            {
                System.out.println(args[i]);
                try
                {
                    finder.listResources(args[i], System.out);
                }
                catch(final IOException ex)
                {
                    ex.printStackTrace();
                }
                System.out.println();
            }
        }
        else
        {
            final LineNumberReader reader = new LineNumberReader(new InputStreamReader(System.in));
            try
            {
                for(String resourceName = JavaResourceFinder.prompt(System.out, JavaResourceFinder.PROMPT, reader); resourceName.length() > 0; resourceName = JavaResourceFinder.prompt(System.out, JavaResourceFinder.PROMPT, reader))
                {
                    finder.listResources(resourceName, System.out);
                }
            }
            catch(final IOException ex)
            {
                ex.printStackTrace();
            }
        }
    }

    protected static String prompt(final PrintStream out, final String prompt, final LineNumberReader reader) throws IOException
    {
        out.print(prompt);
        return reader.readLine().trim();
    }
}
